@@ -8,7 +8,7 @@ use strict;
 use warnings;
 
 our $AUTHORITY = 'cpan:MAROS';
-our $VERSION = 1.38;
+our $VERSION = 1.39;
 
 use MooseX::App::Meta::Role::Attribute::Option;
 use MooseX::App::Exporter qw(app_usage app_description app_base app_fuzzy app_strict app_prefer_commandline app_permute option parameter);
@@ -298,6 +298,13 @@ This is equivalent to
   );
 
 Single letter options are treated as flags and may be combined with eachother.
+However such options must have a Boolean type constraint.
+
+ option 'verbose' => (
+      is            => 'rw',
+      isa           => 'Bool',
+      cmd_flag      => 'v',
+  );
 
 Positional parameters are defined with the 'parameter' keyword
 
