@@ -30,6 +30,13 @@ has 'app_namespace' => (
     builder     => '_build_app_namespace',
 );
 
+has 'app_exclude' => (
+    is          => 'rw',
+    isa         => 'MooseX::App::Types::List',
+    coerce      => 1,
+    default     => sub { [] },
+);
+
 has 'app_base' => (
     is          => 'rw',
     isa         => 'Str',
@@ -93,12 +100,22 @@ sub _build_app_namespace {
 sub _build_app_commands {
     my ($self) = @_;
 
-    my @list;
+    my (@list);
+    # Process namespace list
     foreach my $namespace ( @{ $self->app_namespace } ) {
         push(@list,$self->command_scan_namespace($namespace));
     }
+    my $commands = { @list };
 
-    return { @list };
+    # Process excludes
+    foreach my $exclude ( @{ $self->app_exclude } ) {
+        foreach my $command (keys %{$commands}) {
+            delete $commands->{$command}
+                if $commands->{$command} =~ m/^\Q$exclude\E(::|$)/;
+        }
+    }
+
+    return $commands;
 }
 
 sub command_check {
@@ -125,9 +142,8 @@ sub command_scan_namespace {
     foreach my $command_class ($mpo->plugins) {
         my $command_class_name =  substr($command_class,length($namespace)+2);
 
-        # Check for odd class names - needs to be refactored for subcommands support
-        next
-            if $command_class_name =~ m/::/;
+        # subcommands support
+        $command_class_name =~ s/::/ /g;
 
         # Extract command name
         $command_class_name =~ s/^\Q$namespace\E:://;
@@ -148,6 +164,10 @@ sub command_args {
     $metaclass ||= $self;
     my $parsed_argv = MooseX::App::ParsedArgv->instance;
 
+    unless ($metaclass->does_role('MooseX::App::Role::Common')) {
+        Moose->throw_error('Class '.$metaclass->name.' is not a proper MooseX::App::Command class. You either need to use MooseX::App::Command or exclude this class via app_exclude')
+    }
+
     # Process options
     my @attributes_option = $self->command_usage_attributes($metaclass,'option');
 
@@ -500,49 +520,84 @@ sub command_candidates {
 }
 
 sub command_find {
-    my ($self,$command) = @_;
+    my ($self,$commands) = @_;
 
-    my $lc_command = lc($command);
-    my $commands = $self->app_commands;
+    my $parsed_argv     = MooseX::App::ParsedArgv->instance;
+    my $all_commands    = $self->app_commands;
 
-    # Exact match
-    if (defined $commands->{$lc_command}) {
-        return $lc_command;
-    } else {
-        my $candidate =  $self->command_candidates($command);
-
-        if (ref $candidate eq '') {
-            return $candidate;
+    # Get parts
+    my (@parts,@command_parts);
+    if (defined $commands) {
+        if (ref($commands) eq 'ARRAY') {
+            @parts = map { lc } @{$commands};
         } else {
-            given (scalar @{$candidate}) {
-                when (0) {
+            @parts = ( lc($commands) );
+        }
+    } else {
+        @parts = $parsed_argv->elements_argv;
+    }
+
+    # Extract possible parts
+    foreach my $part (@parts) {
+        # Anyting staring with a dash cannot be a command
+        last
+            if $part =~ m/^-/;
+        push(@command_parts,lc($part));
+    }
+
+    # Shortcut
+    return
+        unless scalar @command_parts;
+
+    # basically do a longest-match search
+    for my $index (reverse(0..$#command_parts)) {
+        my $command = join ' ', @command_parts[0..$index];
+        if( $all_commands->{$command} ) {
+            $parsed_argv->shift_argv for 0..$index;
+            return $command;
+        }
+    }
+
+    # didn't find an exact match, let's go to plan B
+    foreach my $index (reverse(0..$#command_parts)) {
+        my $command     = join ' ', @command_parts[0..$index];
+        my $candidate   = $self->command_candidates($command);
+        if (ref $candidate eq '') {
+            $parsed_argv->shift_argv;
+            return $candidate;
+        }
+        given (scalar @{$candidate}) {
+            when (0) {
+                next;
+            }
+            when (1) {
+                if ($self->app_fuzzy) {
+                    $parsed_argv->shift_argv;
+                    return $candidate->[0];
+                } else {
                     return $self->command_message(
                         header          => "Unknown command '$command'", # LOCALIZE
                         type            => "error",
-                    );
-                }
-                when (1) {
-                    if ($self->app_fuzzy) {
-                        return $candidate->[0];
-                    } else {
-                        return $self->command_message(
-                            header          => "Unknown command '$command'", # LOCALIZE
-                            type            => "error",
-                            body            => "Did you mean '".$candidate->[0]."'?", # LOCALIZE
-                        );
-                    }
-                }
-                default {
-                    return $self->command_message(
-                        header          => "Ambiguous command '$command'", # LOCALIZE
-                        type            => "error",
-                        body            => "Which command did you mean?\n". # LOCALIZE
-                            MooseX::App::Utils::format_list(map { [ $_ ] } sort @{$candidate}),
+                        body            => "Did you mean '".$candidate->[0]."'?", # LOCALIZE
                     );
                 }
             }
+            default {
+                return $self->command_message(
+                    header          => "Ambiguous command '$command'", # LOCALIZE
+                    type            => "error",
+                    body            => "Which command did you mean?\n". # LOCALIZE
+                        MooseX::App::Utils::format_list(map { [ $_ ] } sort @{$candidate}),
+                );
+            }
         }
     }
+
+    my $command = $command_parts[0];
+    return $self->command_message(
+        header          => "Unknown command '$command'", # LOCALIZE
+        type            => "error",
+    );
 }
 
 sub command_parser_hints {
@@ -668,6 +723,10 @@ sub command_usage_attributes {
     $metaclass ||= $self;
     $types ||= [qw(option proto)];
 
+    unless ($metaclass->does_role('MooseX::App::Role::Common')) {
+        Moose->throw_error('Class '.$metaclass->name.' is not a proper MooseX::App::Command class. You either need to use MooseX::App::Command or exclude this class via app_exclude')
+    }
+
     my @return;
     foreach my $attribute ($metaclass->get_all_attributes) {
         next
@@ -810,19 +869,36 @@ sub command_class_to_command {
     return;
 }
 
+sub command_subcommands {
+    my ($self,$command_meta_class) = @_;
+
+    $command_meta_class ||= $self;
+    my $command_class = $command_meta_class->name;
+    my $command_name = $self->command_class_to_command($command_class);
+
+    my $commands    = $self->app_commands;
+    my $subcommands = {};
+    foreach my $command (keys %{$commands}) {
+        next
+            if $command eq $command_name
+            || $command !~ m/^\Q$command_name\E\s(.+)/;
+        $subcommands->{$1} = $commands->{$command};
+    }
+
+    return $subcommands;
+}
+
 sub command_usage_command {
     my ($self,$command_meta_class) = @_;
 
     $command_meta_class ||= $self;
 
-    my $command_class = $command_meta_class->name;
-    my $command_name = $self->command_class_to_command($command_class);
-
     my @usage;
     push(@usage,$self->command_usage_header($command_meta_class));
     push(@usage,$self->command_usage_description($command_meta_class));
     push(@usage,$self->command_usage_parameters($command_meta_class,'parameters:')); # LOCALIZE
     push(@usage,$self->command_usage_options($command_meta_class,'options:')); # LOCALIZE
+    push(@usage,$self->command_usage_subcommands('available subcommands:',$self->command_subcommands($command_meta_class))); # LOCALIZE
 
     return @usage;
 }
@@ -830,24 +906,6 @@ sub command_usage_command {
 sub command_usage_global {
     my ($self) = @_;
 
-    my @commands;
-    push(@commands,['help','Prints this usage information']); # LOCALIZE
-
-    my $commands = $self->app_commands;
-
-    foreach my $command (keys %$commands) {
-        my $class = $commands->{$command};
-        Class::Load::load_class($class);
-        my $command_description;
-        $command_description = $class->meta->command_short_description
-            if $class->meta->can('command_short_description');
-
-        $command_description ||= '';
-        push(@commands,[$command,$command_description]);
-    }
-
-    @commands = sort { $a->[0] cmp $b->[0] } @commands;
-
     my @usage;
     push(@usage,$self->command_usage_header());
 
@@ -856,16 +914,46 @@ sub command_usage_global {
         if $description;
     push(@usage,$self->command_usage_parameters($self,'global parameters:')); # LOCALIZE
     push(@usage,$self->command_usage_options($self,'global options:')); # LOCALIZE
-    push(@usage,
-        $self->command_message(
-            header  => 'available commands:', # LOCALIZE
-            body    => MooseX::App::Utils::format_list(@commands),
-        )
-    );
+    push(@usage,$self->command_usage_subcommands('available commands:',$self->app_commands)); # LOCALIZE
 
     return @usage;
 }
 
+sub command_usage_subcommands {
+    my ($self,$headline,$commands) = @_;
+
+    my @commands;
+
+    foreach my $command (keys %$commands) {
+        my $class = $commands->{$command};
+        Class::Load::load_class($class);
+    }
+
+    foreach my $command (keys %$commands) {
+        my $class = $commands->{$command};
+
+        unless ($class->can('meta')
+            && $class->DOES('MooseX::App::Role::Common')) {
+            Moose->throw_error('Class '.$class.' is not a proper MooseX::App::Command class. You either need to use MooseX::App::Command or exclude this class via app_exclude')
+        }
+
+        my $command_description;
+        $command_description = $class->meta->command_short_description
+            if $class->meta->can('command_short_description');
+
+        $command_description ||= '';
+        push(@commands,[$command,$command_description]);
+    }
+
+    @commands = sort { $a->[0] cmp $b->[0] } @commands;
+    push(@commands,['help','Prints this usage information']); # LOCALIZE
+
+    return $self->command_message(
+        header  => $headline,
+        body    => MooseX::App::Utils::format_list(@commands),
+    );
+}
+
 1;
 
 __END__
@@ -897,6 +985,10 @@ the C<_build_app_messageclass> method. Defaults to MooseX::App::Message::Block
 Usually MooseX::App will take the package name of the base class as the
 namespace for commands. This namespace can be changed.
 
+=head2 app_exclude
+
+Exclude namespaces included in app_namespace
+
 =head2 app_base
 
 Usually MooseX::App will take the name of the calling wrapper script to
@@ -1004,7 +1096,7 @@ Returns a message containing the basic usage documentation
 
 =head2 command_find
 
- my @commands = $meta->command_find($user_command_input);
+ my @commands = $meta->command_find($commands_arrayref);
 
 Returns a list of command names matching the user input
 
