@@ -8,7 +8,7 @@ use strict;
 use warnings;
 
 our $AUTHORITY = 'cpan:MAROS';
-our $VERSION = 1.38;
+our $VERSION = 1.39;
 
 use Moose::Exporter;
 use MooseX::App::Exporter qw(app_usage app_description app_base app_fuzzy app_strict app_prefer_commandline app_permute option parameter command_short_description command_long_description command_usage command_strict);
@@ -76,6 +76,9 @@ sub new_with_options {
         Moose->throw_error('new_with_command got invalid extra arguments');
     }
 
+    $class->meta->command_check()
+        if $ENV{APP_DEVELOPER} || $ENV{HARNESS_ACTIVE};
+
     # Get ARGV
     my $argv = delete $args{ARGV};
     my $parsed_argv;
