@@ -57,8 +57,12 @@ sub class_to_command {
     return
         unless defined $class;
 
-    my @parts = split( /[_\s]+|\b|(?<![A-Z])(?=[A-Z])|(?<=[A-Z])(?=[A-Z][a-z])/, $class );
-    return lc(join('_',@parts));
+    my @commands;
+    foreach my $part (split /\s+/,$class) {
+        my @parts = split( /_+|\b|(?<![A-Z])(?=[A-Z])|(?<=[A-Z])(?=[A-Z][a-z])/, $part );
+        push (@commands,join('_',@parts));
+    }
+    return lc(join(" ",@commands));
 }
 
 # Format output text for fixed screen width
@@ -254,6 +258,9 @@ sub _pod_node_to_text {
                         push (@lines,"\n");
                         ${$indent}--;
                     }
+                    when (qr/head\d/) {
+                        push (@lines,"\n",$node->content,"\n");
+                    }
                 }
             }
         }
